@@ -7,18 +7,17 @@
 import argparse
 import os
 import re
+import types
 import shutil
 import socket
 
 from xml.etree import ElementTree
-
-import youtube_dl
-
-from PIL import Image, ImageDraw
-from youtube_dl import YoutubeDL
 from datetime import datetime
 
+from PIL import Image, ImageDraw
 from cairosvg.surface import PNGSurface
+
+from youtube_dl import YoutubeDL
 from youtube_dl.compat import (
     compat_http_client,
     compat_urllib_error,
@@ -31,20 +30,21 @@ from youtube_dl.utils import (
     DownloadError,
     determine_ext,
 )
-
 from youtube_dl.downloader.common import FileDownloader
-
 from youtube_dl.extractor.common import InfoExtractor
 from youtube_dl.postprocessor.ffmpeg import FFmpegPostProcessorError
 
 from bbb_dl.ffmpeg import FFMPEG
-
 from bbb_dl.version import __version__
 
 _s = lambda p: xpath_with_ns(p, {'svg': 'http://www.w3.org/2000/svg'})
 _x = lambda p: xpath_with_ns(p, {'xlink': 'http://www.w3.org/1999/xlink'})
 
 
+def dummy_to_stderr(self, message):
+    return
+
+
 class Slide:
     def __init__(
         self,
@@ -95,7 +95,7 @@ class BBBDL(InfoExtractor):
                      (?P<id>[0-9a-f\-]+)
                    '''
 
-    def __init__(self, verbose: bool, no_check_certificate: bool, encoder: str, filename: str, audiocodec: str):
+    def __init__(self, verbose: bool, no_check_certificate: bool, encoder: str, audiocodec: str):
         if '_VALID_URL_RE' not in self.__dict__:
             BBBDL._VALID_URL_RE = re.compile(self._VALID_URL)
 
@@ -107,9 +107,18 @@ class BBBDL(InfoExtractor):
             ydl_options.update({"nocheckcertificate": True})
 
         self.verbose = verbose
-        self.ydl = youtube_dl.YoutubeDL(ydl_options)
-        self.set_downloader(self.ydl)
+        self.ydl = YoutubeDL(ydl_options)
         self.ffmpeg = FFMPEG(self.ydl, encoder, audiocodec)
+        super().__init__(self.ydl)
+
+    def _get_automatic_captions(self, *args, **kwargs):
+        return
+
+    def _get_subtitles(self, *args, **kwargs):
+        return
+
+    def _mark_watched(self, *args, **kwargs):
+        return
 
     def run(
         self, dl_url: str, add_webcam: bool, add_annotations: bool, add_cursor, keep_tmp_files: bool, filename: str
@@ -144,9 +153,73 @@ class BBBDL(InfoExtractor):
             if bbb_origin_version is not None:
                 bbb_version = bbb_origin_version.split(' ')[0]
                 self.to_screen("BBB version: " + bbb_version)
-        except Exception:
+        except IndexError:
             pass
 
+        # Downlaoding Webcam / Deskshare
+        video_base_url = video_website + '/presentation/' + video_id
+
+        if not self.verbose:
+            self.ydl.to_stderr_backup = self.ydl.to_stderr
+            self.ydl.to_stderr = types.MethodType(dummy_to_stderr, self.ydl)
+
+        webcams_path = video_id + '/webcams.webm'
+        try:
+            self.to_screen("Downloading webcams.webm")
+            webcams_dl = {
+                'id': video_id,
+                'title': title,
+                'url': video_base_url + '/video/webcams.webm',
+                'timestamp': int(start_time),
+            }
+            self.ydl.params['outtmpl'] = webcams_path
+            self.ydl.process_ie_result(webcams_dl)
+        except DownloadError:
+            self.to_screen("Downloading webcams.webm failed! Downloading webcams.mp4 instead")
+            webcams_path = video_id + '/webcams.mp4'
+            try:
+                webcams_dl = {
+                    'id': video_id,
+                    'title': title,
+                    'url': video_base_url + '/video/webcams.mp4',
+                    'timestamp': int(start_time),
+                }
+                self.ydl.params['outtmpl'] = webcams_path
+                self.ydl.process_ie_result(webcams_dl)
+            except DownloadError:
+                webcams_path = None
+                self.to_screen("Error: Downloading webcams.mp4 failed!")
+
+        deskshare_path = video_id + '/deskshare.webm'
+        try:
+            self.to_screen("Downloading deskshare.webm")
+            deskshare_dl = {
+                'id': video_id,
+                'title': title,
+                'url': video_base_url + '/deskshare/deskshare.webm',
+                'timestamp': int(start_time),
+            }
+            self.ydl.params['outtmpl'] = deskshare_path
+            self.ydl.process_ie_result(deskshare_dl)
+        except DownloadError:
+            self.to_screen("Downloading deskshare.webm failed! Downloading deskshare.mp4 instead")
+            deskshare_path = video_id + '/deskshare.mp4'
+            try:
+                deskshare_dl = {
+                    'id': video_id,
+                    'title': title,
+                    'url': video_base_url + '/deskshare/deskshare.mp4',
+                    'timestamp': int(start_time),
+                }
+                self.ydl.params['outtmpl'] = deskshare_path
+                self.ydl.process_ie_result(deskshare_dl)
+            except DownloadError:
+                deskshare_path = None
+                self.to_screen("Warning: Downloading deskshare.mp4 failed - No desk was likely shared in this session.")
+
+        if not self.verbose:
+            self.ydl.to_stderr = self.ydl.to_stderr_backup
+
         # Downloading Slides
         images = list()
         self.xml_find_rec(shapes, _s('svg:image'), images)
@@ -181,6 +254,7 @@ class BBBDL(InfoExtractor):
             if img_path.endswith('deskshare.png'):
                 image_url = video_website + '/presentation/' + video_id + '/deskshare/deskshare.webm'
                 slide_filename = 'deskshare.webm'
+                slide_path = deskshare_path
             else:
                 if img_path not in img_path_to_filename:
                     slide_filename = 'slide-{:03d}'.format(counter) + '.' + determine_ext(img_path)
@@ -188,8 +262,8 @@ class BBBDL(InfoExtractor):
                     counter += 1
                 else:
                     slide_filename = img_path_to_filename[img_path]
+                slide_path = video_id + '/' + slide_filename
 
-            slide_path = video_id + '/' + slide_filename
             slide_ts_in = float(image.get('in'))
             slide_ts_out = float(image.get('out'))
             slide_ts_duration = max(0.0, min(recording_duration - slide_ts_in, slide_ts_out - slide_ts_in))
@@ -221,61 +295,6 @@ class BBBDL(InfoExtractor):
         if add_cursor:
             slides_infos = self._add_cursor(slides_infos, cursor_infos)
 
-        # Downlaoding Webcam / Deskshare
-        video_base_url = video_website + '/presentation/' + video_id
-
-        webcams_path = video_id + '/webcams.webm'
-        try:
-            self.to_screen("Downloading webcams.webm")
-            webcams_dl = {
-                'id': video_id,
-                'title': title,
-                'url': video_base_url + '/video/webcams.webm',
-                'timestamp': int(start_time),
-            }
-            self.ydl.params['outtmpl'] = webcams_path
-            self.ydl.process_ie_result(webcams_dl)
-        except DownloadError:
-            webcams_path = video_id + '/webcams.mp4'
-            try:
-                self.to_screen("Downloading webcams.mp4")
-                webcams_dl = {
-                    'id': video_id,
-                    'title': title,
-                    'url': video_base_url + '/video/webcams.mp4',
-                    'timestamp': int(start_time),
-                }
-                self.ydl.params['outtmpl'] = webcams_path
-                self.ydl.process_ie_result(webcams_dl)
-            except DownloadError:
-                webcams_path = None
-
-        deskshare_path = video_id + '/deskshare.webm'
-        try:
-            self.to_screen("Downloading deskshare.webm")
-            deskshare_dl = {
-                'id': video_id,
-                'title': title,
-                'url': video_base_url + '/deskshare/deskshare.webm',
-                'timestamp': int(start_time),
-            }
-            self.ydl.params['outtmpl'] = deskshare_path
-            self.ydl.process_ie_result(deskshare_dl)
-        except DownloadError:
-            deskshare_path = video_id + '/deskshare.mp4'
-            try:
-                self.to_screen("Downloading deskshare.mp4")
-                deskshare_dl = {
-                    'id': video_id,
-                    'title': title,
-                    'url': video_base_url + '/deskshare/deskshare.mp4',
-                    'timestamp': int(start_time),
-                }
-                self.ydl.params['outtmpl'] = deskshare_path
-                self.ydl.process_ie_result(deskshare_dl)
-            except DownloadError:
-                deskshare_path = None
-
         # Post processing
         slideshow_w, slideshow_h = self._rescale_slides(slides_infos)
 
@@ -363,8 +382,7 @@ class BBBDL(InfoExtractor):
                 real_ts_in = None
                 ignore_original_slide = False
                 draw_elements = slide.annotations.findall(_s("./svg:g[@timestamp]"))
-                for i in range(len(draw_elements)):
-                    draw_elm = draw_elements[i]
+                for i, draw_elm in enumerate(draw_elements):
                     ts_in = int(float(draw_elm.get('timestamp')))
 
                     if ts_in <= int(slide.ts_in):
@@ -441,7 +459,7 @@ class BBBDL(InfoExtractor):
         cursors = cursor_infos.findall("event[@timestamp]")
         cursors[0].attrib['timestamp'] = '0.0'
 
-        for cursor_id in range(len(cursors)):
+        for cursor_id, cursor in enumerate(cursors):
             cursor = cursors[cursor_id]
 
             ts_in = int(float(cursor.get('timestamp')))
@@ -644,7 +662,7 @@ class BBBDL(InfoExtractor):
         slideshow_path = video_id + '/slideshow.mp4'
 
         video_list = video_id + '/video_list.txt'
-        vl_file = open(video_list, 'w')
+        vl_file = open(video_list, 'w', encoding="utf-8")
 
         self.to_screen("Create slideshow")
         for i, slide in enumerate(slides_infos):
@@ -789,7 +807,7 @@ def main(args=None):
     parser = get_parser()
     args = parser.parse_args(args)
 
-    BBBDL(args.verbose, args.no_check_certificate, args.encoder, args.filename, args.audiocodec).run(
+    BBBDL(args.verbose, args.no_check_certificate, args.encoder, args.audiocodec).run(
         args.URL,
         args.add_webcam,
         args.add_annotations,
